@@ -29,10 +29,11 @@ the Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA
    For GNU Fortran (g77), we do the following to the argument list
    before passing it to `gcc':
 
-   1.  Put `-xf77' or `-xf77-cpp-input' before each list of foo.f or foo.F
-       source files and put `-xnone' after that list, if necessary.
-       This shouldn't normally be necessary, but it is done in case
-       gcc.c normally treats .f/.F files as, say, to be compiled by f2c.
+   1.  Put `-xf77', `-xf77-cpp-input' or `-xratfor' before each list
+       of foo.f, foo.F or foo.r source files and put `-xnone' after
+       that list, if necessary.  This shouldn't normally be necessary,
+       but it is done in case gcc.c normally treats .f/.F files as,
+       say, to be compiled by f2c.
 
    2.  Make sure `-lf2c -lm' is at the end of the list.
 
@@ -252,6 +253,7 @@ typedef enum
   OPTION_o,			/* Aka --output. */
   OPTION_P,			/* Aka --print-*-name. */
   OPTION_S,			/* Aka --assemble. */
+  OPTION_syntax_only,		/* -fsyntax-only. */
   OPTION_v,			/* Aka --verbose. */
   OPTION_version,		/* --version. */
   OPTION_V,			/* Aka --use-version. */
@@ -779,9 +781,7 @@ lookup_option (xopt, xskip, xarg, text)
 	   "iwithprefix", "iwithprefixbefore", "isystem".  */
 	;
 
-      if (text[1] != '-')
-	skip = 0;
-      else if (strcmp (text, "--assemble") == 0)
+      if (strcmp (text, "--assemble") == 0)
 	opt = OPTION_S;
       else if (strcmp (text, "--compile") == 0)
 	opt = OPTION_c;
@@ -823,6 +823,8 @@ lookup_option (xopt, xskip, xarg, text)
 	       || (strcmp (text, "--no-standard-libraries") == 0)
 	       || (strcmp (text, "-nodefaultlibs") == 0))
 	opt = OPTION_nostdlib;
+      else if (strcmp (text, "-fsyntax-only") == 0)
+	opt = OPTION_syntax_only;
       else if (opteq (&skip, &arg, text, "--use-version") == 0)
 	opt = OPTION_V;
       else if (strcmp (text, "--verbose") == 0)
@@ -1165,7 +1167,8 @@ main (argc, argv)
   /* 0 => -xnone in effect on input/output
      1 => -xfoo in effect on input/output
      2 => -xnone in effect on input, -xf77 on output
-     3 => -xnone in effect on input, -xf77-cpp-input on output.  */
+     3 => -xnone in effect on input, -xf77-cpp-input on output.
+     4 => -xnone in effect on input, -xratfor on output.  */
   int saw_speclang = 0;
 
   /* 0 => initial/reset state
@@ -1238,6 +1241,7 @@ main (argc, argv)
 	case OPTION_nostdlib:
 	case OPTION_c:
 	case OPTION_S:
+	case OPTION_syntax_only:
 	case OPTION_E:
 	case OPTION_M:
 	case OPTION_MM:
@@ -1260,7 +1264,7 @@ main (argc, argv)
 
 	case OPTION_v:
 	  if (!verbose)
-	    printf ("g77 version %s\n", ffezzz_version_string);
+	    fprintf (stderr, "g77 version %s\n", ffezzz_version_string);
 	  verbose = 1;
 	  break;
 
@@ -1277,7 +1281,7 @@ main (argc, argv)
 	case OPTION_version:
 	  printf ("\
 GNU Fortran %s\n\
-Copyright (C) 1996 Free Software Foundation, Inc.\n\
+Copyright (C) 1997 Free Software Foundation, Inc.\n\
 For more version information on components of the GNU Fortran\n\
 compilation system, especially useful when reporting bugs,\n\
 type the command `g77 --verbose'.\n\
@@ -1400,20 +1404,43 @@ Report bugs to fortran@gnu.ai.mit.edu.\n");
 	  else if ((argv[i][0] != '-')	/* Not a library. */
 		   && (len = strlen (argv[i])) > 2
 		   && ((argv[i][len - 1] == 'F')
-		       || (argv[i][len - 1] == 'f'))
+		       || (argv[i][len - 1] == 'f')
+		       || (argv[i][len - 1] == 'r'))
 		   && argv[i][len - 2] == '.')
-	    {			/* filename.f or filename.F. */
+	    {			/* filename.f or filename.F. or filename.r */
 	      if (saw_library == 1)	/* -l<library>. */
 		append_arg ("-lm");
 	      saw_library = 0;
-	      want_speclang = (argv[i][len - 1] == 'F') + 2;
-
+	      switch (argv[i][len - 1])
+		{
+		case 'f':
+		  want_speclang = 2;
+		  break;
+		case 'F':
+		  want_speclang = 3;
+		  break;
+		case 'r':
+		  want_speclang = 4;
+		  break;
+		default:
+		  break;
+		}
 	      if (saw_speclang != want_speclang)
 		{
-		  if (want_speclang == 2)
-		    append_arg ("-xf77");
-		  else
-		    append_arg ("-xf77-cpp-input");
+		  switch (want_speclang)
+		    {
+		    case 2:
+		      append_arg ("-xf77");
+		      break;
+		    case 3:
+		      append_arg ("-xf77-cpp-input");
+		      break;
+		    case 4:
+		      append_arg ("-xratfor");
+		      break;
+		    default:
+		      break;
+		    }
 		  saw_speclang = want_speclang;
 		}
 	    }
