@@ -19,10 +19,31 @@ along with GNU Fortran; see the file COPYING.  If not, write to
 the Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA
 02111-1307, USA.  */
 
-#include "proj.h"
+/* From f/proj.h, which uses #error -- not all C compilers
+   support that, and we want _this_ program to be compilable
+   by pretty much any C compiler.  */
+
+#include "assert.j"		/* Use gcc's assert.h. */
+#include <stdio.h>
+#include <stddef.h>
+#include <stdlib.h>
+#include <string.h>
 #define FFEINTRIN_DOC 1
 #include "intrin.h"
 
+typedef enum
+  {
+#if !defined(false) || !defined(true)
+    false = 0, true = 1,
+#endif
+#if !defined(FALSE) || !defined(TRUE)
+    FALSE = 0, TRUE = 1,
+#endif
+    Doggone_Trailing_Comma_Dont_Work = 1
+  } bool;
+
+#define ARRAY_SIZE(a) (sizeof(a)/sizeof(a[0]))
+
 char *family_name (ffeintrinFamily family);
 static void dumpif (ffeintrinFamily fam);
 static void dumpendif (void);
@@ -32,8 +53,8 @@ static void dumpgen (int menu, char *name, char *name_uc,
 		     ffeintrinGen gen);
 static void dumpspec (int menu, char *name, char *name_uc,
 		      ffeintrinSpec spec);
-static void dumpimp (int menu, char *name, char *name_uc,
-		     size_t genno, ffeintrinFamily family, ffeintrinImp imp);
+static void dumpimp (int menu, char *name, char *name_uc, size_t genno, ffeintrinFamily family,
+		     ffeintrinImp imp, ffeintrinSpec spec);
 static char *argument_info_ptr (ffeintrinImp imp, int argno);
 static char *argument_info_string (ffeintrinImp imp, int argno);
 static char *argument_name_ptr (ffeintrinImp imp, int argno);
@@ -51,8 +72,8 @@ main (int argc, char **argv __attribute__ ((unused)))
   if (argc != 1)
     {
       fprintf (stderr, "\
-Usage: intdoc > intdoc.texi
-  Collects and dumps documentation on g77 intrinsics
+Usage: intdoc > intdoc.texi\n\
+  Collects and dumps documentation on g77 intrinsics\n\
   to the file named intdoc.texi.\n");
       exit (1);
     }
@@ -89,7 +110,6 @@ struct _ffeintrin_spec_
 struct _ffeintrin_imp_
   {
     char *name;			/* Name of implementation. */
-    ffeintrinImp cg_imp;	/* Unique code-generation code. */
 #if 0	/* FFECOM_targetCURRENT == FFECOM_targetGCC */
     ffecomGfrt gfrt;		/* gfrt index in library. */
 #endif	/* FFECOM_targetCURRENT == FFECOM_targetGCC */
@@ -101,14 +121,12 @@ static struct _ffeintrin_name_ names[] = {
   { UPPER, LOWER, MIXED, FFEINTRIN_ ## GEN, FFEINTRIN_ ## SPEC },
 #define DEFGEN(CODE,NAME,SPEC1,SPEC2)
 #define DEFSPEC(CODE,NAME,CALLABLE,FAMILY,IMP)
-#define DEFIMP(CODE,NAME,GFRT,CONTROL)
-#define DEFIMQ(CODE,NAME,GFRT,CONTROL,CGIMP)
+#define DEFIMP(CODE,NAME,GFRTDIRECT,GFRTF2C,GFRTGNU,CONTROL)
 #include "intrin.def"
 #undef DEFNAME
 #undef DEFGEN
 #undef DEFSPEC
 #undef DEFIMP
-#undef DEFIMQ
 };
 
 static struct _ffeintrin_gen_ gens[] = {
@@ -116,14 +134,12 @@ static struct _ffeintrin_gen_ gens[] = {
 #define DEFGEN(CODE,NAME,SPEC1,SPEC2) \
   { NAME, { SPEC1, SPEC2, }, },
 #define DEFSPEC(CODE,NAME,CALLABLE,FAMILY,IMP)
-#define DEFIMP(CODE,NAME,GFRT,CONTROL)
-#define DEFIMQ(CODE,NAME,GFRT,CONTROL,CGIMP)
+#define DEFIMP(CODE,NAME,GFRTDIRECT,GFRTF2C,GFRTGNU,CONTROL)
 #include "intrin.def"
 #undef DEFNAME
 #undef DEFGEN
 #undef DEFSPEC
 #undef DEFIMP
-#undef DEFIMQ
 };
 
 static struct _ffeintrin_imp_ imps[] = {
@@ -131,15 +147,11 @@ static struct _ffeintrin_imp_ imps[] = {
 #define DEFGEN(CODE,NAME,SPEC1,SPEC2)
 #define DEFSPEC(CODE,NAME,CALLABLE,FAMILY,IMP)
 #if 0	/* FFECOM_targetCURRENT == FFECOM_targetGCC */
-#define DEFIMP(CODE,NAME,GFRT,CONTROL) \
-  { NAME, FFEINTRIN_imp ## CODE, FFECOM_gfrt ## GFRT, CONTROL },
-#define DEFIMQ(CODE,NAME,GFRT,CONTROL,CGIMP) \
-  { NAME, FFEINTRIN_imp ## CGIMP, FFECOM_gfrt ## GFRT, CONTROL },
+#define DEFIMP(CODE,NAME,GFRTDIRECT,GFRTF2C,GFRTGNU,CONTROL) \
+  { NAME, FFECOM_gfrt ## GFRT, CONTROL },
 #elif 1	/* FFECOM_targetCURRENT == FFECOM_targetFFE */
-#define DEFIMP(CODE,NAME,GFRT,CONTROL) \
-  { NAME, FFEINTRIN_imp ## CODE, CONTROL },
-#define DEFIMQ(CODE,NAME,GFRT,CONTROL,CGIMP) \
-  { NAME, FFEINTRIN_imp ## CGIMP, CONTROL },
+#define DEFIMP(CODE,NAME,GFRTDIRECT,GFRTF2C,GFRTGNU,CONTROL) \
+  { NAME, CONTROL },
 #else
 #error
 #endif
@@ -148,7 +160,6 @@ static struct _ffeintrin_imp_ imps[] = {
 #undef DEFGEN
 #undef DEFSPEC
 #undef DEFIMP
-#undef DEFIMQ
 };
 
 static struct _ffeintrin_spec_ specs[] = {
@@ -156,24 +167,26 @@ static struct _ffeintrin_spec_ specs[] = {
 #define DEFGEN(CODE,NAME,SPEC1,SPEC2)
 #define DEFSPEC(CODE,NAME,CALLABLE,FAMILY,IMP) \
   { NAME, CALLABLE, FAMILY, IMP, },
-#define DEFIMP(CODE,NAME,GFRT,CONTROL)
-#define DEFIMQ(CODE,NAME,GFRT,CONTROL,CGIMP)
+#define DEFIMP(CODE,NAME,GFRTDIRECT,GFRTF2C,GFRTGNU,CONTROL)
 #include "intrin.def"
 #undef DEFGEN
 #undef DEFSPEC
 #undef DEFIMP
-#undef DEFIMQ
 };
 
-static char *descriptions[FFEINTRIN_imp] = {
-#define DEFDOC(IMP,SUMMARY,DESCRIPTION) [FFEINTRIN_imp ## IMP] DESCRIPTION,
-#include "intdoc.h"
+struct cc_pair { ffeintrinImp imp; char *text; };
+
+static char *descriptions[FFEINTRIN_imp] = { 0 };
+static struct cc_pair cc_descriptions[] = {
+#define DEFDOC(IMP,SUMMARY,DESCRIPTION) { FFEINTRIN_imp ## IMP, DESCRIPTION },
+#include "intdoc.h0"
 #undef DEFDOC
 };
 
-static char *summaries[FFEINTRIN_imp] = {
-#define DEFDOC(IMP,SUMMARY,DESCRIPTION) [FFEINTRIN_imp ## IMP] SUMMARY,
-#include "intdoc.h"
+static char *summaries[FFEINTRIN_imp] = { 0 };
+static struct cc_pair cc_summaries[] = {
+#define DEFDOC(IMP,SUMMARY,DESCRIPTION) { FFEINTRIN_imp ## IMP, SUMMARY },
+#include "intdoc.h0"
 #undef DEFDOC
 };
 
@@ -209,6 +222,9 @@ family_name (ffeintrinFamily family)
     case FFEINTRIN_familyF2U:
       return "familyF2U";
 
+    case FFEINTRIN_familyBADU77:
+      return "familyBADU77";
+
     default:
       assert ("bad family" == NULL);
       return "??";
@@ -254,6 +270,20 @@ dumpem ()
 {
   int i;
 
+  for (i = 0; ((size_t) i) < ARRAY_SIZE (cc_descriptions); ++i)
+    {
+      assert (descriptions[cc_descriptions[i].imp] == NULL);
+      descriptions[cc_descriptions[i].imp] = cc_descriptions[i].text;
+    }
+
+  for (i = 0; ((size_t) i) < ARRAY_SIZE (cc_summaries); ++i)
+    {
+      assert (summaries[cc_summaries[i].imp] == NULL);
+      summaries[cc_summaries[i].imp] = cc_summaries[i].text;
+    }
+
+  printf ("@c This file is automatically derived from intdoc.c, intdoc.in,\n");
+  printf ("@c ansify.c, intrin.def, and intrin.h.  Edit those files instead.\n");
   printf ("@menu\n");
   for (i = 0; ((size_t) i) < ARRAY_SIZE (names); ++i)
     {
@@ -284,19 +314,51 @@ static void
 dumpgen (int menu, char *name, char *name_uc, ffeintrinGen gen)
 {
   size_t i;
+  int total = 0;
+
+  if (!menu)
+    {
+      for (i = 0; i < ARRAY_SIZE (gens[gen].specs); ++i)
+	{
+	  if (gens[gen].specs[i] != FFEINTRIN_specNONE)
+	    ++total;
+	}
+    }
 
   for (i = 0; i < ARRAY_SIZE (gens[gen].specs); ++i)
     {
       ffeintrinSpec spec;
+      size_t j;
 
       if ((spec = gens[gen].specs[i]) == FFEINTRIN_specNONE)
 	continue;
 
-      if (specs[spec].implementation == FFEINTRIN_impNONE)
-	continue;
-
       dumpif (specs[spec].family);
-      dumpimp (menu, name, name_uc, i, specs[spec].family, specs[spec].implementation);
+      dumpimp (menu, name, name_uc, i, specs[spec].family, specs[spec].implementation,
+	       spec);
+      if (!menu && (total > 0))
+	{
+	  if (total == 1)
+	    {
+	      printf ("\
+For information on another intrinsic with the same name:\n");
+	    }
+	  else
+	    {
+	      printf ("\
+For information on other intrinsics with the same name:\n");
+	    }
+	  for (j = 0; j < ARRAY_SIZE (gens[gen].specs); ++j)
+	    {
+	      if (j == i)
+		continue;
+	      if ((spec = gens[gen].specs[j]) == FFEINTRIN_specNONE)
+		continue;
+	      printf ("@xref{%s Intrinsic (%s)}.\n",
+		      name, specs[spec].name);
+	    }
+	  printf ("\n");
+	}
       dumpendif ();
     }
 }
@@ -304,45 +366,53 @@ dumpgen (int menu, char *name, char *name_uc, ffeintrinGen gen)
 static void
 dumpspec (int menu, char *name, char *name_uc, ffeintrinSpec spec)
 {
-  if (specs[spec].implementation == FFEINTRIN_impNONE)
-    return;
-
   dumpif (specs[spec].family);
-  dumpimp (menu, name, name_uc, 0, specs[spec].family, specs[spec].implementation);
+  dumpimp (menu, name, name_uc, 0, specs[spec].family, specs[spec].implementation,
+	   FFEINTRIN_specNONE);
   dumpendif ();
 }
 
 static void
-dumpimp (int menu, char *name, char *name_uc, size_t genno, ffeintrinFamily family, ffeintrinImp imp)
+dumpimp (int menu, char *name, char *name_uc, size_t genno, ffeintrinFamily family, ffeintrinImp imp,
+	 ffeintrinSpec spec)
 {
-  char *c = imps[imp].control;
-  bool subr = (c[0] == '-');
+  char *c;
+  bool subr;
   char *argc;
   char *argi;
-  int colon = (c[2] == ':') ? 2 : 3;
+  int colon;
   int argno;
 
+  assert ((imp != FFEINTRIN_impNONE) || !genno);
+
   if (menu)
     {
       printf ("* %s Intrinsic",
 	      name);
-      if (genno)
-	printf (" (Form %s)", imps[imp].name);
+      if (spec != FFEINTRIN_specNONE)
+	printf (" (%s)", specs[spec].name);	/* See XYZZY1 below */
       printf ("::");
-      if (summaries[imp] != NULL)
-	{
 #define INDENT_SUMMARY 24
+      if ((imp == FFEINTRIN_impNONE)
+	  || (summaries[imp] != NULL))
+	{
 	  int spaces = INDENT_SUMMARY - 14 - strlen (name);
-	  char *c = summaries[imp];
+	  char *c;
 
-	  if (genno != 0)
-	    spaces -= (8 + strlen (imps[imp].name));
+	  if (spec != FFEINTRIN_specNONE)
+	    spaces -= (3 + strlen (specs[spec].name));	/* See XYZZY1 above */
 	  if (spaces < 1)
 	    spaces = 1;
 	  while (spaces--)
 	    fputc (' ', stdout);
 
-	  while (c[0] != '\0')
+	  if (imp == FFEINTRIN_impNONE)
+	    {
+	      printf ("(Reserved for future use.)\n");
+	      return;
+	    }
+
+	  for (c = summaries[imp]; c[0] != '\0'; ++c)
 	    {
 	      if ((c[0] == '@')
 		  && (c[1] >= '0')
@@ -372,7 +442,6 @@ dumpimp (int menu, char *name, char *name_uc, size_t genno, ffeintrinFamily fami
 		}
 	      else
 		fputc (c[0], stdout);
-	      ++c;
 	    }
 	}
       printf ("\n");
@@ -380,16 +449,36 @@ dumpimp (int menu, char *name, char *name_uc, size_t genno, ffeintrinFamily fami
     }
 
   printf ("@node %s Intrinsic", name);
-  if (genno)
-    printf (" (Form %s)", imps[imp].name);
+  if (spec != FFEINTRIN_specNONE)
+    printf (" (%s)", specs[spec].name);
   printf ("\n@subsubsection %s Intrinsic", name);
-  if (genno)
-    printf (" (Form %s)", imps[imp].name);
-  printf ("\n@cindex %s intrinsic\n@cindex intrinsics, %s\n
-@noindent
-@example
+  if (spec != FFEINTRIN_specNONE)
+    printf (" (%s)", specs[spec].name);
+  printf ("\n@cindex %s intrinsic\n@cindex intrinsics, %s\n",
+	  name, name);
+
+  if (imp == FFEINTRIN_impNONE)
+    {
+      printf ("\n\
+This intrinsic is not yet implemented.\n\
+The name is, however, reserved as an intrinsic.\n\
+Use @samp{EXTERNAL %s} to use this name for an\n\
+external procedure.\n\
+\n\
+",
+	      name);
+      return;
+    }
+
+  c = imps[imp].control;
+  subr = (c[0] == '-');
+  colon = (c[2] == ':') ? 2 : 3;
+
+  printf ("\n\
+@noindent\n\
+@example\n\
 %s%s(",
-	  name, name, (subr ? "CALL " : ""), name);
+	  (subr ? "CALL " : ""), name);
 
   fflush (stdout);
 
@@ -410,8 +499,9 @@ dumpimp (int menu, char *name, char *name_uc, size_t genno, ffeintrinFamily fami
 		argc, argc);
     }
 
-  printf (")
-@end example\n
+  printf (")\n\
+@end example\n\
+\n\
 ");
 
   if (!subr)
@@ -435,7 +525,7 @@ dumpimp (int menu, char *name, char *name_uc, size_t genno, ffeintrinFamily fami
 	}
 
       printf ("\
-@noindent
+@noindent\n\
 %s: ", name);
       print_type_string (c);
       printf (" function");
@@ -449,20 +539,20 @@ dumpimp (int menu, char *name, char *name_uc, size_t genno, ffeintrinFamily fami
 	  || (arg_info[0] == '*') || (arg_info[0] == 'n') || (arg_info[0] == 'p'))
 	    ++arg_info;
 	  if ((arg_info[0] == 'F') || (arg_info[0] == 'N'))
-	    printf (".
-The exact type is @samp{REAL(KIND=1)} when argument @var{%s} is
-any type other than @code{COMPLEX}, or when it is @code{COMPLEX(KIND=1)}.
-When @var{%s} is any @code{COMPLEX} type other than @code{COMPLEX(KIND=1)},
-this intrinsic is valid only when used as the argument to
+	    printf (".\n\
+The exact type is @samp{REAL(KIND=1)} when argument @var{%s} is\n\
+any type other than @code{COMPLEX}, or when it is @code{COMPLEX(KIND=1)}.\n\
+When @var{%s} is any @code{COMPLEX} type other than @code{COMPLEX(KIND=1)},\n\
+this intrinsic is valid only when used as the argument to\n\
 @code{REAL()}, as explained below.\n\n",
 		    arg_string,
 		    arg_string);
 	  else
-	    printf (".
-This intrinsic is valid when argument @var{%s} is
-@code{COMPLEX(KIND=1)}.
-When @var{%s} is any other @code{COMPLEX} type,
-this intrinsic is valid only when used as the argument to
+	    printf (".\n\
+This intrinsic is valid when argument @var{%s} is\n\
+@code{COMPLEX(KIND=1)}.\n\
+When @var{%s} is any other @code{COMPLEX} type,\n\
+this intrinsic is valid only when used as the argument to\n\
 @code{REAL()}, as explained below.\n\n",
 		    arg_string,
 		    arg_string);
@@ -470,7 +560,7 @@ this intrinsic is valid only when used as the argument to
 #if 0
       else if ((c[0] == 'I')
 	       && (c[1] == 'p'))
-	printf (", the exact type being wide enough to hold a pointer
+	printf (", the exact type being wide enough to hold a pointer\n\
 on the target system (typically @code{INTEGER(KIND=1)} or @code{INTEGER(KIND=4)}).\n\n");
 #endif
       else if ((c[1] == '=')
@@ -496,10 +586,10 @@ on the target system (typically @code{INTEGER(KIND=1)} or @code{INTEGER(KIND=4)}
 		   && ((arg_info[0] == 'C')
 		       || (arg_info[0] == 'F')
 		       || (arg_info[0] == 'N')))
-	    printf (".
-The exact type depends on that of argument @var{%s}---if @var{%s} is
-@code{COMPLEX}, this function's type is @code{REAL}
-with the same @samp{KIND=} value as the type of @var{%s}.
+	    printf (".\n\
+The exact type depends on that of argument @var{%s}---if @var{%s} is\n\
+@code{COMPLEX}, this function's type is @code{REAL}\n\
+with the same @samp{KIND=} value as the type of @var{%s}.\n\
 Otherwise, this function's type is the same as that of @var{%s}.\n\n",
 		    arg_string, arg_string, arg_string, arg_string);
 	  else
@@ -508,7 +598,7 @@ Otherwise, this function's type is the same as that of @var{%s}.\n\n",
 	}
       else if ((c[1] == '=')
 	       && (c[colon + 1] == '*'))
-	printf (", the exact type being the result of cross-promoting the
+	printf (", the exact type being the result of cross-promoting the\n\
 types of all the arguments.\n\n");
       else if (c[1] == '=')
 	assert ("?0:?:" == NULL);
@@ -526,7 +616,7 @@ types of all the arguments.\n\n");
       int elements;
 
       printf ("\
-@noindent
+@noindent\n\
 @var{");
       for (; ; ++argc)
 	{
@@ -788,12 +878,12 @@ types of all the arguments.\n\n");
 	  break;
 
 	case 'g':
-	  printf ("@samp{*@var{label}}, where @var{label} is the label
+	  printf ("@samp{*@var{label}}, where @var{label} is the label\n\
 of an executable statement");
 	  break;
 
 	case 's':
-	  printf ("Signal handler (@code{INTEGER FUNCTION} or @code{SUBROUTINE})
+	  printf ("Signal handler (@code{INTEGER FUNCTION} or @code{SUBROUTINE})\n\
 or dummy/global @code{INTEGER(KIND=1)} scalar");
 	  break;
 
@@ -878,7 +968,7 @@ or dummy/global @code{INTEGER(KIND=1)} scalar");
     }
 
   printf ("\
-@noindent
+@noindent\n\
 Intrinsic groups: ");
   switch (family)
     {
@@ -918,6 +1008,10 @@ Intrinsic groups: ");
       printf ("@code{unix}.");
       break;
 
+    case FFEINTRIN_familyBADU77:
+      printf ("@code{badu77}.");
+      break;
+
     default:
       assert ("bad family" == NULL);
       printf ("@code{???}.");
@@ -930,8 +1024,8 @@ Intrinsic groups: ");
       char *c = descriptions[imp];
 
       printf ("\
-@noindent
-Description:
+@noindent\n\
+Description:\n\
 \n");
 
       while (c[0] != '\0')
