@@ -70,8 +70,8 @@ the Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   "-fno-ugly-init",
   "-fugly-logint",
   "-fno-ugly-logint",
-  "-fdebug",
-  "-fno-debug",
+  "-fxyzzy",
+  "-fno-xyzzy",
   "-finit-local-zero",
   "-fno-init-local-zero",
   "-fbackslash",
@@ -135,8 +135,12 @@ the Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   "-fno-onetrip",
   "-fsilent",
   "-fno-silent",
+  "-fglobals",
+  "-fno-globals",
   "-ftypeless-boz",
   "-fno-typeless-boz",
+  "-Wglobals",
+  "-Wno-globals",
 /*"-Wimplicit",*/
 /*"-Wno-implicit",*/
   "-Wsurprising",
