@@ -1,9 +1,9 @@
-static char junk[] = "\n@(#)LIBF77 VERSION 19960619\n";
+static char junk[] = "\n@(#)LIBF77 VERSION 19970404\n";
 
 /*
 */
 
-char __G77_LIBF77_VERSION__[] = "0.5.20";
+char __G77_LIBF77_VERSION__[] = "0.5.21";
 
 /*
 2.00	11 June 1980.  File version.c added to library.
@@ -47,6 +47,12 @@ char __G77_LIBF77_VERSION__[] = "0.5.20";
 	19 Mar. 1996: s_cat.c: supply missing break after overlap detection.
 	13 May 1996:  add [lq]bitbits.c and [lq]bitshft.c (f90 bit intrinsics).
 	19 June 1996: add casts to unsigned in [lq]bitshft.c.
+	26 Feb. 1997: adjust functions with a complex output argument
+			to permit aliasing it with input arguments.
+			(For now, at least, this is just for possible
+			benefit of g77.)
+	4 April 1997: [cz]_div.c: tweaks invisible on most systems (that may
+			affect systems using gratuitous extra precision).
 */
 
 #include <stdio.h>
