@@ -4,11 +4,11 @@
 #ifdef KR_headers
 extern VOID sig_die();
 
-int abort_()
+int G77_abort_0 ()
 #else
 extern void sig_die(char*,int);
 
-int abort_(void)
+int G77_abort_0 (void)
 #endif
 {
 sig_die("Fortran abort routine called", 1);
