@@ -1,9 +1,9 @@
-static char junk[] = "\n@(#) LIBI77 VERSION pjw,dmg-mods 19961209\n";
+static char junk[] = "\n@(#) LIBI77 VERSION pjw,dmg-mods 19970816\n";
 
 /*
 */
 
-char __G77_LIBI77_VERSION__[] = "0.5.20";
+char __G77_LIBI77_VERSION__[] = "0.5.21";
 
 /*
 2.01	$ format added
@@ -245,6 +245,26 @@ wrtfmt.c:
 /* 9 Dec. 1996:	 d[fu]e.c, err.c: complain about non-positive rec=
 		 in direct read and write statements.
 		 ftell_.c: change param "unit" to "Unit" for -DKR_headers. */
+/* 26 Feb. 1997: ftell_.c: on systems that define SEEK_SET, etc., use
+		 SEEK_SET, SEEK_CUR, SEEK_END for *whence = 0, 1, 2. */
+/* 7 Apr. 1997:	 fmt.c: adjust to complain at missing numbers in formats
+		 (but still treat missing ".nnn" as ".0"). */
+/* 11 Apr. 1997: err.c: attempt to make stderr line buffered rather
+		 than fully buffered.  (Buffering is needed for format
+		 items T and TR.) */
+/* 27 May 1997:  ftell_.c: fix typo (that caused the third argument to be
+		 treated as 2 on some systems). */
+/* 5 Aug. 1997:  lread.c: adjust to accord with a change to the Fortran 8X
+		 draft (in 1990 or 1991) that rescinded permission to elide
+		 quote marks in namelist input of character data; compile
+		 with -DF8X_NML_ELIDE_QUOTES to get the old behavior.
+		 wrtfmt.o: wrt_G: tweak to print the right number of 0's
+		 for zero under G format. */
+/* 16 Aug. 1997: iio.c: fix bug in internal writes to an array of character
+		 strings that sometimes caused one more array element than
+		 required by the format to be blank-filled.  Example:
+		 format(1x). */
+/* 17 June 1997: detect recursive I/O and call f__fatal explaining it. */
 
 #include <stdio.h>
 
