@@ -14,6 +14,8 @@ integer f_back(alist *a)
 #endif
 	long x, y;
 	char buf[32];
+	if (f__init & 2)
+		f__fatal (131, "I/O recursion");
 	if(a->aunit >= MXUNIT || a->aunit < 0)
 		err(a->aerr,101,"backspace");
 	b= &f__units[a->aunit];
