@@ -27,6 +27,8 @@ integer f_clos(cllist *a)
 #endif
 {	unit *b;
 
+	if (f__init & 2)
+		f__fatal (131, "I/O recursion");
 	if(a->cunit >= MXUNIT) return(0);
 	b= &f__units[a->cunit];
 	if(b->ufd==NULL)
@@ -71,6 +73,8 @@ f_exit(void)
 #endif
 {	int i;
 	static cllist xx;
+	if (! (f__init & 1))
+		return;		/* Not initialized, so no open units. */
 	if (!xx.cerr) {
 		xx.cerr=1;
 		xx.csta=NULL;
@@ -83,9 +87,9 @@ f_exit(void)
 }
  int
 #ifdef KR_headers
-flush_()
+G77_flush_0 ()
 #else
-flush_(void)
+G77_flush_0 (void)
 #endif
 {	int i;
 	for(i=0;i<MXUNIT;i++)
