@@ -3,9 +3,6 @@
 #include <sys/stat.h>
 #endif
 #include "f2c.h"
-#include "fio.h"
-#include "fmt.h"	/* for struct syl */
-#include "rawio.h"	/* for fcntl.h, fdopen */
 #if defined (NON_UNIX_STDIO) || defined (MISSING_FILE_ELEMS)
 #ifdef KR_headers
 extern char *malloc();
@@ -16,10 +13,15 @@ extern char *malloc();
 #include <stdlib.h>
 #endif
 #endif
+#include "fio.h"
+#include "fmt.h"	/* for struct syl */
+#include "rawio.h"	/* for fcntl.h, fdopen */
 
 /*global definitions*/
 unit f__units[MXUNIT];	/*unit table*/
-flag f__init;	/*0 on entry, 1 after initializations*/
+int f__init;	/*bit 0: set after initializations;
+		  bit 1: set during I/O involving returns to
+		    caller of library (or calls to user code)*/
 cilist *f__elist;	/*active external io list*/
 icilist *f__svic;	/*active internal io list*/
 flag f__reading;	/*1 if reading, 0 if writing*/
@@ -76,7 +78,8 @@ char *F_err[] =
 	"can't write file",				/* 127 */
 	"'new' file exists",				/* 128 */
 	"can't append to file",				/* 129 */
-	"non-positive record number"			/* 130 */
+	"non-positive record number",			/* 130 */
+	"I/O started while already doing I/O"		/* 131 */
 };
 #define MAXERR (sizeof(F_err)/sizeof(char *)+100)
 
@@ -141,6 +144,8 @@ f__fatal(n,s) char *s;
 f__fatal(int n, char *s)
 #endif
 {
+	static int dead = 0;
+
 	if(n<100 && n>=0) perror(s); /*SYSDEP*/
 	else if(n >= (int)MAXERR || n < -1)
 	{	fprintf(stderr,"%s: illegal error number %d\n",s,n);
@@ -148,18 +153,26 @@ f__fatal(int n, char *s)
 	else if(n == -1) fprintf(stderr,"%s: end of file\n",s);
 	else
 		fprintf(stderr,"%s: %s\n",s,F_err[n-100]);
-	if (f__curunit) {
-		fprintf(stderr,"apparent state: unit %d ",f__curunit-f__units);
-		fprintf(stderr, f__curunit->ufnm ? "named %s\n" : "(unnamed)\n",
-			f__curunit->ufnm);
-		}
-	else
-		fprintf(stderr,"apparent state: internal I/O\n");
-	if (f__fmtbuf)
-		fprintf(stderr,"last format: %s\n",f__fmtbuf);
-	fprintf(stderr,"lately %s %s %s %s",f__reading?"reading":"writing",
-		f__sequential?"sequential":"direct",f__formatted?"formatted":"unformatted",
-		f__external?"external":"internal");
+	if (dead) {
+		fprintf (stderr, "(libf2c f__fatal already called, aborting.)");
+		abort();
+	}
+	dead = 1;
+	if (f__init & 1) {
+		if (f__curunit) {
+			fprintf(stderr,"apparent state: unit %d ",f__curunit-f__units);
+			fprintf(stderr, f__curunit->ufnm ? "named %s\n" : "(unnamed)\n",
+				f__curunit->ufnm);
+			}
+		else
+			fprintf(stderr,"apparent state: internal I/O\n");
+		if (f__fmtbuf)
+			fprintf(stderr,"last format: %s\n",f__fmtbuf);
+		fprintf(stderr,"lately %s %s %s %s",f__reading?"reading":"writing",
+			f__sequential?"sequential":"direct",f__formatted?"formatted":"unformatted",
+			f__external?"external":"internal");
+	}
+	f__init &= ~2;	/* No longer doing I/O (no more user code to be called). */
 	sig_die(" IO", 1);
 }
 /*initialization routine*/
@@ -167,14 +180,20 @@ f__fatal(int n, char *s)
 f_init(Void)
 {	unit *p;
 
-	f__init=1;
+	if (f__init & 2)
+		f__fatal (131, "I/O recursion");
+	f__init = 1;
 	p= &f__units[0];
 	p->ufd=stderr;
 	p->useek=f__canseek(stderr);
+#ifdef _IOLBF
+	setvbuf(stderr, (char*)malloc(BUFSIZ+8), _IOLBF, BUFSIZ+8);
+#else
 #if defined (NON_UNIX_STDIO) || defined (MISSING_FILE_ELEMS)
-	setbuf(stderr, (char *)malloc(BUFSIZ));
+	setbuf(stderr, (char *)malloc(BUFSIZ+8));
 #else
 	stderr->_flag &= ~_IONBF;
+#endif
 #endif
 	p->ufmt=1;
 	p->uwrt=1;
@@ -274,5 +293,6 @@ err__fl(int f, int m, char *s)
 		f__fatal(m, s);
 	if (f__doend)
 		(*f__doend)();
+	f__init &= ~2;
 	return errno = m;
 	}
