@@ -1,9 +1,11 @@
+#include <ctype.h>
 #include "f2c.h"
 #include "fio.h"
-#include "fmt.h"
-#include "lio.h"
-#include <ctype.h>
-#include "fp.h"
+
+/* Compile with -DF8X_NML_ELIDE_QUOTES to permit eliding quotation */
+/* marks in namelist input a la the Fortran 8X Draft published in  */
+/* the May 1989 issue of Fortran Forum. */
+
 
 extern char *f__fmtbuf;
 
@@ -24,6 +26,11 @@ int (*f__lioproc)(), (*l_getc)(), (*l_ungetc)();
 int (*f__lioproc)(ftnint*, char*, ftnlen, ftnint), (*l_getc)(void),
 	(*l_ungetc)(int,FILE*);
 #endif
+
+#include "fmt.h"
+#include "lio.h"
+#include "fp.h"
+
 int l_eof;
 
 #define isblnk(x) (f__ltab[x+1]&B)
@@ -78,6 +85,7 @@ t_getc(Void)
 integer e_rsle(Void)
 {
 	int ch;
+	f__init = 1;
 	if(f__curunit->uend) return(0);
 	while((ch=t_getc())!='\n')
 		if (ch == EOF) {
@@ -92,7 +100,7 @@ flag f__lquit;
 int f__lcount,f__ltype,nml_read;
 char *f__lchar;
 double f__lx,f__ly;
-#define ERR(x) if(n=(x)) return(n)
+#define ERR(x) if(n=(x)) {f__init &= ~2; return(n);}
 #define GETC(x) (x=(*l_getc)())
 #define Ungetc(x,y) (*l_ungetc)(x,y)
 
@@ -385,6 +393,10 @@ l_CHAR(Void)
 				case '*':
 					if (f__lcount == 0) {
 						f__lcount = 1;
+#ifndef F8X_NML_ELIDE_QUOTES
+						if (nml_read)
+							goto no_quote;
+#endif
 						goto noquote;
 						}
 					p = f__lchar;
@@ -403,6 +415,13 @@ l_CHAR(Void)
 				}
 			if (!isdigit(ch)) {
 				f__lcount = 1;
+#ifndef F8X_NML_ELIDE_QUOTES
+				if (nml_read) {
+ no_quote:
+					errfl(f__elist->cierr,112,
+						"undelimited character string");
+					}
+#endif
 				goto noquote;
 				}
 			*p++ = ch;
@@ -419,10 +438,17 @@ l_CHAR(Void)
 	else	(void) Ungetc(ch,f__cf);
  have_lcount:
 	if(GETC(ch)=='\'' || ch=='"') quote=ch;
-	else if(isblnk(ch) || (issep(ch) && ch != '\n') || ch==EOF)
-	{	(void) Ungetc(ch,f__cf);
-		return(0);
-	}
+	else if(isblnk(ch) || (issep(ch) && ch != '\n') || ch==EOF) {
+		Ungetc(ch,f__cf);
+		return 0;
+		}
+#ifndef F8X_NML_ELIDE_QUOTES
+	else if (nml_read > 1) {
+		Ungetc(ch,f__cf);
+		f__lquit = 2;
+		return 0;
+		}
+#endif
 	else {
 		/* Fortran 8x-style unquoted string */
 		*p++ = ch;
@@ -489,8 +515,8 @@ c_le(a) cilist *a;
 c_le(cilist *a)
 #endif
 {
-	if(!f__init)
-		f_init();
+	if(f__init != 1) f_init();
+	f__init = 3;
 	f__fmtbuf="list io";
 	if(a->ciunit>=MXUNIT || a->ciunit<0)
 		err(a->cierr,101,"stler");
@@ -556,8 +582,7 @@ l_read(ftnint *number, char *ptr, ftnlen len, ftnint type)
 			quad_read = 1;
 			n = l_R(0);
 			quad_read = 0;
-			if (n)
-				return n;
+			ERR(n);
 			break;
 #endif
 		case TYCOMPLEX:
