@@ -1,4 +1,4 @@
-/* Copyright (C) 1995 Free Software Foundation, Inc.
+/* Copyright (C) 1995, 1997 Free Software Foundation, Inc.
 This file is part of GNU Fortran libU77 library.
 
 This library is free software; you can redistribute it and/or modify it
@@ -33,11 +33,15 @@ Boston, MA 02111-1307, USA.  */
 
 
 #ifdef KR_headers
-integer chdir_ (name, Lname)
+void g_char ();
+
+integer G77_chdir_0 (name, Lname)
      char *name;
      ftnlen Lname;
 #else
-integer chdir_ (const char *name, const ftnlen  Lname)
+void g_char(const char *a, ftnlen alen, char *b);
+
+integer G77_chdir_0 (const char *name, const ftnlen Lname)
 #endif
 {
   char *buff;
@@ -46,10 +50,7 @@ integer chdir_ (const char *name, const ftnlen  Lname)
 
   buff = malloc (Lname+1);
   if (buff == NULL) return -1;
-  blast = buff + Lname;
-  for (bp = buff ; bp<blast ; )
-    *bp++ = *name++;
-  *bp = '\0';
+  g_char (name, Lname, buff);
   i = chdir (buff);
   free (buff);
   return i ? errno : 0;
