@@ -19,26 +19,28 @@ Boston, MA 02111-1307, USA.  */
 #include "f2c.h"
 #include <math.h>
 
-doublereal dbesj0_ (const double *x) {
+#if 0 	/* Don't include these unless necessary -- dnp. */
+double G77_dbesj0_0 (const double *x) {
     return j0 (*x);
 }
 
-doublereal dbesj1_ (const double *x) {
+double G77_dbesj1_0 (const double *x) {
     return j1 (*x);
 }
 
-doublereal dbesjn_ (const integer *n, double *x) {
+double G77_dbesjn_0 (const integer *n, double *x) {
      return jn (*n, *x);
  }
 
-doublereal dbesy0_ (const double *x) {
+double G77_dbesy0_0 (const double *x) {
     return y0 (*x);
 }
 
-doublereal dbesy1_ (const double *x) {
+double G77_dbesy1_0 (const double *x) {
     return y1 (*x);
 }
 
-doublereal dbesyn_ (const integer *n, double *x) {
+double G77_dbesyn_0 (const integer *n, double *x) {
     return yn (*n, *x);
 }
+#endif
