@@ -26,9 +26,9 @@ Boston, MA 02111-1307, USA.  */
 #include "f2c.h"
 
 #ifdef KR_headers
-integer getgid_ ()
+integer G77_getgid_0 ()
 #else
-integer getgid_ (void)
+integer G77_getgid_0 (void)
 #endif
 {
   return getgid ();
