@@ -26,9 +26,9 @@ Boston, MA 02111-1307, USA.  */
 #include "f2c.h"
 
 #ifdef KR_headers
-integer getuid_ ()
+integer G77_getuid_0 ()
 #else
-integer getuid_ (void)
+integer G77_getuid_0 (void)
 #endif
 {
   return getuid ();
