@@ -27,12 +27,14 @@ Boston, MA 02111-1307, USA.  */
 #if HAVE_UNISTD_H
 #  include <unistd.h>
 #endif
+#include <errno.h>		/* for ENOSYS */
 #include "f2c.h"
 
-integer hostnm_(char *name, ftnlen Lname)
+integer G77_hostnm_0 (char *name, ftnlen Lname)
 {
     int ret, i;
 
+#if HAVE_GETHOSTNAME
     ret = gethostname (name, Lname);
     if (ret==0) {
 	/* Pad with blanks (assuming gethostname will make an error
@@ -41,4 +43,7 @@ integer hostnm_(char *name, ftnlen Lname)
 	    name[i] = ' ';
     }
     return ret;
+#else
+    return errno = ENOSYS;
+#endif
 }
