@@ -1802,6 +1802,16 @@ ffesta_set_outpooldisp (ffestaPooldisp d)
   ffesta_outpooldisp_ = d;
 }
 
+/* Shut down current parsing possibility, but without bothering the
+   user with a diagnostic if we're not inhibited.  */
+
+void
+ffesta_shutdown ()
+{
+  if (ffesta_is_inhibited_)
+    ffesta_current_shutdown_ = TRUE;
+}
+
 /* ffesta_two -- Deal with the first two tokens after a swallowed statement
 
    return ffesta_two(first_token,second_token);	 // to lexer.
