@@ -529,6 +529,9 @@ ffestorag_update_init (ffestorag s)
       && !ffesymbol_is_init (s->symbol))
     ffesymbol_update_init (s->symbol);
 
+  if (s->parent != NULL)
+    ffestorag_update_init (s->parent);
+
   for (sq = s->equivs_.first;
        sq != (ffestorag) &s->equivs_.first;
        sq = ffestorag_next_ (sq))
@@ -557,6 +560,9 @@ ffestorag_update_save (ffestorag s)
       && !ffesymbol_is_save (s->symbol))
     ffesymbol_update_save (s->symbol);
 
+  if (s->parent != NULL)
+    ffestorag_update_save (s->parent);
+
   for (sq = s->equivs_.first;
        sq != (ffestorag) &s->equivs_.first;
        sq = ffestorag_next_ (sq))
