@@ -1,5 +1,5 @@
 /* stu.c -- Implementation File (module.c template V1.0)
-   Copyright (C) 1995, 1996 Free Software Foundation, Inc.
+   Copyright (C) 1995-1997 Free Software Foundation, Inc.
    Contributed by James Craig Burley (burley@gnu.ai.mit.edu).
 
 This file is part of GNU Fortran.
@@ -27,6 +27,7 @@ the Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA
 #include "bld.h"
 #include "com.h"
 #include "equiv.h"
+#include "global.h"
 #include "info.h"
 #include "implic.h"
 #include "intrin.h"
@@ -113,8 +114,81 @@ ffestu_sym_end_transition (ffesymbol s)
       if ((swh == FFEINFO_whereLOCAL)
 	  && ((skd == FFEINFO_kindFUNCTION)
 	      || (skd == FFEINFO_kindSUBROUTINE)))
-	ffestu_dummies_transition_ (ffecom_sym_end_transition,
-				    ffesymbol_dummyargs (s));
+	{
+	  int n_args;
+	  ffebld list;
+	  ffebld item;
+	  ffeglobalArgSummary as;
+	  ffeinfoBasictype bt;
+	  ffeinfoKindtype kt;
+	  bool array;
+	  char *name = NULL;
+
+	  ffestu_dummies_transition_ (ffecom_sym_end_transition,
+				      ffesymbol_dummyargs (s));
+
+	  n_args = ffebld_list_length (ffesymbol_dummyargs (s));
+	  ffeglobal_proc_def_nargs (s, n_args);
+	  for (list = ffesymbol_dummyargs (s), n_args = 0;
+	       list != NULL;
+	       list = ffebld_trail (list), ++n_args)
+	    {
+	      item = ffebld_head (list);
+	      array = FALSE;
+	      if (item != NULL)
+		{
+		  bt = ffeinfo_basictype (ffebld_info (item));
+		  kt = ffeinfo_kindtype (ffebld_info (item));
+		  array = (ffeinfo_rank (ffebld_info (item)) > 0);
+		  switch (ffebld_op (item))
+		    {
+		    case FFEBLD_opSTAR:
+		      as = FFEGLOBAL_argsummaryALTRTN;
+		      break;
+
+		    case FFEBLD_opSYMTER:
+		      name = ffesymbol_text (ffebld_symter (item));
+		      as = FFEGLOBAL_argsummaryNONE;
+
+		      switch (ffeinfo_kind (ffebld_info (item)))
+			{
+			case FFEINFO_kindFUNCTION:
+			  as = FFEGLOBAL_argsummaryFUNC;
+			  break;
+
+			case FFEINFO_kindSUBROUTINE:
+			  as = FFEGLOBAL_argsummarySUBR;
+			  break;
+
+			case FFEINFO_kindNONE:
+			  as = FFEGLOBAL_argsummaryPROC;
+			  break;
+
+			default:
+			  break;
+			}
+
+		      if (as != FFEGLOBAL_argsummaryNONE)
+			break;
+
+		      /* Fall through.  */
+		    default:
+		      if (bt == FFEINFO_basictypeCHARACTER)
+			as = FFEGLOBAL_argsummaryDESCR;
+		      else
+			as = FFEGLOBAL_argsummaryREF;
+		      break;
+		    }
+		}
+	      else
+		{
+		  as = FFEGLOBAL_argsummaryNONE;
+		  bt = FFEINFO_basictypeNONE;
+		  kt = FFEINFO_kindtypeNONE;
+		}
+	      ffeglobal_proc_def_arg (s, n_args, name, as, bt, kt, array);
+	    }
+	}
       else if (swh == FFEINFO_whereDUMMY)
 	{
 	  if (ffesymbol_numentries (s) == 0)
@@ -143,8 +217,8 @@ ffestu_sym_end_transition (ffesymbol s)
       assert ("bad status" == NULL);
       return s;
     }
-  ns = FFESYMBOL_stateUNDERSTOOD;
 
+  ns = FFESYMBOL_stateUNDERSTOOD;
   na = sa = ffesymbol_attrs (s);
 
   assert (!(sa & ~(FFESYMBOL_attrsACTUALARG
@@ -265,6 +339,7 @@ ffestu_sym_end_transition (ffesymbol s)
 	  ffesymbol_set_state (s, FFESYMBOL_stateUNDERSTOOD);
 	  ffesymbol_resolve_intrin (s);
 	  s = ffecom_sym_learned (s);
+	  ffesymbol_reference (s, NULL, FALSE);
 	  ffestorag_end_layout (s);
 	  ffesymbol_signal_unreported (s);	/* For debugging purposes. */
 	  return s;
@@ -299,9 +374,8 @@ ffestu_sym_end_transition (ffesymbol s)
       else
 	ffesymbol_resolve_intrin (s);
       s = ffecom_sym_learned (s);
+      ffesymbol_reference (s, NULL, FALSE);
       ffestorag_end_layout (s);
-      if (nwh == FFEINFO_whereGLOBAL)
-	ffesymbol_globalize (s);
       ffesymbol_signal_unreported (s);	/* For debugging purposes. */
     }
 
@@ -368,6 +442,7 @@ ffestu_sym_exec_transition (ffesymbol s)
 	    ffesymbol_error (s, ffesta_tokens[0]);
 	}
 
+      ffesymbol_reference (s, NULL, FALSE);
       ffestorag_exec_layout (s);
       ffesymbol_signal_unreported (s);	/* For debugging purposes. */
       return s;
@@ -719,6 +794,7 @@ ffestu_sym_exec_transition (ffesymbol s)
 					   FFEINFO_whereINTRINSIC,
 					   FFETARGET_charactersizeNONE));
 	  ffesymbol_resolve_intrin (s);
+	  ffesymbol_reference (s, NULL, FALSE);
 	  ffestorag_exec_layout (s);
 	  ffesymbol_signal_unreported (s);	/* For debugging purposes. */
 	  return s;
@@ -730,6 +806,7 @@ ffestu_sym_exec_transition (ffesymbol s)
 	  ffesymbol_signal_change (s);	/* Can't touch this. */
 	  ffesymbol_set_state (s, FFESYMBOL_stateUNDERSTOOD);
 	  ffesymbol_resolve_intrin (s);
+	  ffesymbol_reference (s, NULL, FALSE);
 	  ffestorag_exec_layout (s);
 	  ffesymbol_signal_unreported (s);	/* For debugging purposes. */
 	  return s;
@@ -802,7 +879,9 @@ ffestu_sym_exec_transition (ffesymbol s)
 
   if (na == FFESYMBOL_attrsetNONE)
     ffesymbol_error (s, ffesta_tokens[0]);
-  else if (!(na & FFESYMBOL_attrsANY))
+  else if (!(na & FFESYMBOL_attrsANY)
+	   && (needs_type || (nkd != skd) || (nwh != swh)
+	       || (na != sa) || (ns != ss)))
     {
       ffesymbol_signal_change (s);
       ffesymbol_set_attrs (s, na);	/* Establish new info. */
@@ -821,9 +900,8 @@ ffestu_sym_exec_transition (ffesymbol s)
 	ffesymbol_error (s, ffesta_tokens[0]);
       else if (resolve_intrin)
 	ffesymbol_resolve_intrin (s);
+      ffesymbol_reference (s, NULL, FALSE);
       ffestorag_exec_layout (s);
-      if (nwh == FFEINFO_whereGLOBAL)
-	ffesymbol_globalize (s);
       ffesymbol_signal_unreported (s);	/* For debugging purposes. */
     }
 
