@@ -1,5 +1,5 @@
 /* Implementation of Fortran symbol manager
-   Copyright (C) 1995, 1996 Free Software Foundation, Inc.
+   Copyright (C) 1995-1997 Free Software Foundation, Inc.
    Contributed by James Craig Burley (burley@gnu.ai.mit.edu).
 
 This file is part of GNU Fortran.
@@ -862,13 +862,6 @@ ffesymbol_error (ffesymbol s, ffelexToken t)
   ffesymbol_signal_unreported (s);
 }
 
-void
-ffesymbol_globalize (ffesymbol s)
-{
-  if (ffesymbol_global (s) == NULL)
-    ffesymbol_set_global (s, ffeglobal_promoted (s));
-}
-
 void
 ffesymbol_init_0 ()
 {
@@ -930,6 +923,90 @@ ffesymbol_lookup_local (ffelexToken t)
   return s;			/* May be NULL here, too. */
 }
 
+/* Registers the symbol as one that is referenced by the
+   current program unit.  Currently applies only to
+   symbols known to have global interest (globals and
+   intrinsics).
+
+   s is the (global/intrinsic) symbol referenced; t is the
+   referencing token; explicit is TRUE if the reference
+   is, e.g., INTRINSIC FOO.  */
+
+void
+ffesymbol_reference (ffesymbol s, ffelexToken t, bool explicit)
+{
+  ffename gn;
+  ffesymbol gs = NULL;
+  ffeinfoKind kind;
+  ffeinfoWhere where;
+  bool okay;
+
+  if (ffesymbol_retractable_)
+    return;
+
+  if (t == NULL)
+    t = ffename_token (s->name);	/* Use the first reference in this program unit. */
+
+  kind = ffesymbol_kind (s);
+  where = ffesymbol_where (s);
+
+  if (where == FFEINFO_whereINTRINSIC)
+    {
+      ffeglobal_ref_intrinsic (s, t,
+			       explicit
+			       || s->explicit_where
+			       || ffeintrin_is_standard (s->generic, s->specific));
+      return;
+    }
+
+  if ((where != FFEINFO_whereGLOBAL)
+      && ((where != FFEINFO_whereLOCAL)
+	  || ((kind != FFEINFO_kindFUNCTION)
+	      && (kind != FFEINFO_kindSUBROUTINE))))
+    return;
+
+  gn = ffename_lookup (ffesymbol_global_, t);
+  if (gn != NULL)
+    gs = ffename_symbol (gn);
+  if ((gs != NULL) && (gs != s))
+    {
+      /* We have just discovered another global symbol with the same name
+	 but a different `nature'.  Complain.  Note that COMMON /FOO/ can
+	 coexist with local symbol FOO, e.g. local variable, just not with
+	 CALL FOO, hence the separate namespaces.  */
+
+      ffesymbol_error (gs, t);
+      ffesymbol_error (s, NULL);
+      return;
+    }
+
+  switch (kind)
+    {
+    case FFEINFO_kindBLOCKDATA:
+      okay = ffeglobal_ref_blockdata (s, t);
+      break;
+
+    case FFEINFO_kindSUBROUTINE:
+      okay = ffeglobal_ref_subroutine (s, t);
+      break;
+
+    case FFEINFO_kindFUNCTION:
+      okay = ffeglobal_ref_function (s, t);
+      break;
+
+    case FFEINFO_kindNONE:
+      okay = ffeglobal_ref_external (s, t);
+      break;
+
+    default:
+      assert ("bad kind in global ref" == NULL);
+      return;
+    }
+
+  if (! okay)
+    ffesymbol_error (s, NULL);
+}
+
 /* Report info on the symbol for debugging purposes.  */
 
 ffesymbol
