@@ -75,6 +75,7 @@ bool ffe_is_f2c_ = FFETARGET_defaultIS_F2C;
 bool ffe_is_f2c_library_ = FFETARGET_defaultIS_F2C_LIBRARY;
 bool ffe_is_ffedebug_ = FALSE;
 bool ffe_is_free_form_ = FFETARGET_defaultIS_FREE_FORM;
+bool ffe_is_globals_ = TRUE;
 bool ffe_is_ident_ = TRUE;
 bool ffe_is_init_local_zero_ = FFETARGET_defaultIS_INIT_LOCAL_ZERO;
 bool ffe_is_mainprog_;		/* TRUE if current prog unit known to be
@@ -93,6 +94,7 @@ bool ffe_is_ugly_init_ = TRUE;
 bool ffe_is_ugly_logint_ = FALSE;
 bool ffe_is_version_ = FALSE;
 bool ffe_is_vxt_ = FALSE;
+bool ffe_is_warn_globals_ = TRUE;
 bool ffe_is_warn_implicit_ = FALSE;
 bool ffe_is_warn_surprising_ = FALSE;
 bool ffe_is_zeros_ = FALSE;
@@ -100,6 +102,7 @@ ffeCase ffe_case_intrin_ = FFETARGET_defaultCASE_INTRIN;
 ffeCase ffe_case_match_ = FFETARGET_defaultCASE_MATCH;
 ffeCase ffe_case_source_ = FFETARGET_defaultCASE_SOURCE;
 ffeCase ffe_case_symbol_ = FFETARGET_defaultCASE_SYMBOL;
+ffeIntrinsicState ffe_intrinsic_state_badu77_ = FFE_intrinsicstateENABLED;
 ffeIntrinsicState ffe_intrinsic_state_gnu_ = FFE_intrinsicstateENABLED;
 ffeIntrinsicState ffe_intrinsic_state_f2c_ = FFE_intrinsicstateENABLED;
 ffeIntrinsicState ffe_intrinsic_state_f90_ = FFE_intrinsicstateENABLED;
@@ -291,9 +294,9 @@ ffe_decode_option (char *opt)
 	ffe_set_is_ugly_logint (TRUE);
       else if (strcmp (&opt[2], "no-ugly-logint") == 0)
 	ffe_set_is_ugly_logint (FALSE);
-      else if (strcmp (&opt[2], "debug") == 0)
+      else if (strcmp (&opt[2], "xyzzy") == 0)
 	ffe_set_is_ffedebug (TRUE);
-      else if (strcmp (&opt[2], "no-debug") == 0)
+      else if (strcmp (&opt[2], "no-xyzzy") == 0)
 	ffe_set_is_ffedebug (FALSE);
       else if (strcmp (&opt[2], "init-local-zero") == 0)
 	ffe_set_is_init_local_zero (TRUE);
@@ -331,6 +334,10 @@ ffe_decode_option (char *opt)
 	ffe_set_is_silent (TRUE);
       else if (strcmp (&opt[2], "no-silent") == 0)
 	ffe_set_is_silent (FALSE);
+      else if (strcmp (&opt[2], "globals") == 0)
+	ffe_set_is_globals (TRUE);
+      else if (strcmp (&opt[2], "no-globals") == 0)
+	ffe_set_is_globals (FALSE);
       else if (strcmp (&opt[2], "typeless-boz") == 0)
 	ffe_set_is_typeless_boz (TRUE);
       else if (strcmp (&opt[2], "no-typeless-boz") == 0)
@@ -407,6 +414,14 @@ ffe_decode_option (char *opt)
 	  ffe_set_case_source (FFE_caseNONE);
 	  ffe_set_case_symbol (FFE_caseNONE);
 	}
+      else if (strcmp (&opt[2], "badu77-intrinsics-delete") == 0)
+	ffe_set_intrinsic_state_badu77 (FFE_intrinsicstateDELETED);
+      else if (strcmp (&opt[2], "badu77-intrinsics-hide") == 0)
+	ffe_set_intrinsic_state_badu77 (FFE_intrinsicstateHIDDEN);
+      else if (strcmp (&opt[2], "badu77-intrinsics-disable") == 0)
+	ffe_set_intrinsic_state_badu77 (FFE_intrinsicstateDISABLED);
+      else if (strcmp (&opt[2], "badu77-intrinsics-enable") == 0)
+	ffe_set_intrinsic_state_badu77 (FFE_intrinsicstateENABLED);
       else if (strcmp (&opt[2], "gnu-intrinsics-delete") == 0)
 	ffe_set_intrinsic_state_gnu (FFE_intrinsicstateDELETED);
       else if (strcmp (&opt[2], "gnu-intrinsics-hide") == 0)
@@ -488,6 +503,10 @@ ffe_decode_option (char *opt)
 	; /* cpp handles this one.  */
       else if (!strcmp (&opt[2], "no-import"))
 	; /* cpp handles this one.  */
+      else if (!strcmp (&opt[2], "globals"))
+	ffe_set_is_warn_globals (TRUE);
+      else if (!strcmp (&opt[2], "no-globals"))
+	ffe_set_is_warn_globals (FALSE);
       else if (!strcmp (&opt[2], "implicit"))
 	ffe_set_is_warn_implicit (TRUE);
       else if (!strcmp (&opt[2], "no-implicit"))
